import datetime
import hashlib
import time
import uuid
from functools import wraps

from paramiko.client import SSHClient, AutoAddPolicy

from app.settings import SECRET_KEY, SIMPLE_JWT

cache_data = {}


def cache(lifetime=300):
    '''
    装饰器：系统缓存
    @Cache(lifetime=60)
    lifetime单位秒，缓存时间
    '''

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            key = get_md5(func.__module__ + func.__name__)
            if cache_data.get(key):
                delta = datetime.timedelta(seconds=cache_data[key]['lifetime'])
                now = datetime.datetime.now()
                if compare_datetime(cache_data[key]['createtime'] + delta, now):
                    return cache_data[key]['data']
            data = func(*args, **kwargs)
            cache_data[key] = {'lifetime': lifetime, 'createtime': datetime.datetime.now(), 'data': data}
            return data

        return wrapper

    return decorator


def get_md5(s):
    m = hashlib.md5()
    m.update(s.encode(encoding='UTF-8'))
    return m.hexdigest()


def get_uuid():
    return uuid.uuid4().hex


def get_timestamp():
    return int(time.time())


def compare_datetime(d1, d2):
    if d1 > d2:
        return True
    elif d1 == d2:
        return None
    else:
        return False


def get_auth(content='', secret=SECRET_KEY):
    timestamp = get_timestamp()
    skey = content + secret + str(timestamp)
    return hashlib.sha256(skey.encode()).hexdigest(), timestamp


def verify_auth(auth, timestamp, content='', secret=SECRET_KEY):
    delta = SIMPLE_JWT['ACCESS_TOKEN_LIFETIME']
    now = datetime.datetime.now()
    t = datetime.datetime.fromtimestamp(timestamp)
    if compare_datetime(now, t + delta):
        return None  # token过期
    skey = content + secret + str(timestamp)
    if auth == hashlib.sha256(skey.encode(encoding='UTF-8')).hexdigest():
        return True  # token验证通过
    else:
        return False  # token验证失败


def build_tree(data):
    node_dict = {item['id']: item for item in data}
    node_list = []
    for item in data:
        item.setdefault('children', [])
        parent_item = node_dict.get(item['parent'])
        if parent_item:
            children = parent_item.setdefault('children', [])
            children.append(item)
        else:
            node_list.append(item)
    return node_list


class RemoteExec(SSHClient):
    nfs_sftp = None

    def __init__(self, agent=None, ip=None, port=None, username=None, password=None):
        super().__init__()
        if agent:
            ip, port, username, password = agent.ip, agent.port, agent.username, agent.password
        self.set_missing_host_key_policy(AutoAddPolicy())
        self.connect(hostname=ip, port=port, username=username, password=password)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.nfs_sftp:
            self.nfs_sftp.close()
        self.close()

    def exec(self, command):
        command = command.strip()
        if any([command.endswith(" &"), command.endswith(" 2>&1")]):  # 有一个为true都pass
            pass
        else:
            command = command + " 2>&1"
        stdin, stdout, stderr = self.exec_command(command)
        return stdout.read().decode().strip()

    def get_sftp(self):
        if self.nfs_sftp is None:
            self.nfs_sftp = self.open_sftp()
        return self.nfs_sftp

    def file_a(self, file_path, file_content, mode='a'):
        ftp = self.get_sftp()
        file = ftp.file(file_path, mode)
        file.write(file_content)
        file.flush()


log_path = '/migrate.log'
split_lines = '*' * 100 + '\n' + '*' * 100


class MigrateRemoteExec(RemoteExec):
    def exec(self, command):
        command = command.strip()
        if any([command.endswith(" &"), command.endswith(" 2>&1")]):  # 有一个为true都pass
            pass
        else:
            command = command + " 2>&1"
        stdin, stdout, stderr = self.exec_command(command)
        res = stdout.read().decode().strip()
        content = f"执行: {command}\n{res}\n{split_lines}\n"
        self.file_a(log_path, content)
        return res

    def exec_pty(self, command):
        command = command.strip()
        if any([command.endswith(" &"), command.endswith(" 2>&1")]):  # 有一个为true都pass
            pass
        else:
            command = command + " 2>&1"
        stdin, stdout, stderr = self.exec_command(command, get_pty=True)
        self.file_a(log_path, f"执行: {command}")
        res = ''
        while not stdout.channel.exit_status_ready():
            line = stdout.readline().strip() + '\n'
            res += line
            self.file_a(log_path, line)
            if stdout.channel.exit_status_ready():
                line = stdout.readline().strip() + '\n'
                res += line
                self.file_a(log_path, line)
                break
        self.file_a(log_path, f"\n{split_lines}\n")
        return res
